#include <stdlib.h>
#include <string.h>
#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h> 

#include "main.h"
#include "lcd.h"
#include "key.h"
#include "menu.c"
#include "valout.h"

volatile uint8_t cur_mode = 0;

void lcd_uint16_unit( const uint16_t v, const char unit ) {
    if ( v == 0xffff ) {
        lcd_puts_p( p_off );
    } else {
        char s[6];
        // FIXME: auf uint anpassen.
        valout( v, 6, 3, s );
        lcd_puts(s);
    }
    lcd_putc( unit );
}

void lcd_itoa10(uint8_t c) {
    char itoabuffer[4];
    itoa(c, itoabuffer, 10);
    for ( int i = strlen( itoabuffer ); i < 3; i++ ) {
        lcd_putc(' ');
    }
    lcd_puts(itoabuffer);
}

void lcd_itoa2(uint8_t c) {
    char itoabuffer[9];
    itoa(c, itoabuffer, 2);
    for ( int i = strlen( itoabuffer ); i < 8; i++ ) {
        lcd_putc('0');
    } 
    lcd_puts(itoabuffer);
}

int main (void) {
    // taster eingang + pullup
    DDRD &= ~((1 << DDD2) | (1 << DDD4) | (1 << DDD7));
    PORTD |= (1 << PD2) | (1 << PD4) | (1 << PD7);

    // led
    LED_DDR |= (1<<LED_PIN);
    LED_PORT |= (1<<LED_PIN);

    // lcd start
    lcd_init(LCD_DISP_ON);
    lcd_clrscr();

    // enable interrupt/timer
    TCCR0B = (1<<CS02) | ( 1<<CS00);        // divide by 1024
    TIMSK0 = 1<<TOIE0;                       // enable timer interrupt
    sei();

    // greeting
    lcd_puts_P("HELLO\n");
    for(uint8_t s = 0; s<=7; s++) {
        lcd_putc(s);
    }
    _delay_ms(1000);
    LED_PORT &= ~(1<<LED_PIN);

    lcd_clrscr();


    uint8_t display = 0;
    static uint8_t menuepos = 2;

    while(1) {
        // menue system
        switch( display ) {
            case DISPLAY_NORM_INIT:
                display = display_norm_init();
            case DISPLAY_NORM_WAITKEY:
                display = display_norm_waitkey();
            break;

            case DISPLAY_MENU_INIT:
                display = display_menu_init( &menuepos );
            case DISPLAY_MENU_CHANGED:
                display = display_menu_changed( &menuepos );
            case DISPLAY_MENU_WAITKEY:
                display = display_menu_waitkey( &menuepos );
            break;

            case DISPLAY_VAL_INIT:
                display = display_val_init( &menuepos );
            case DISPLAY_VAL_CHANGED:
                display = display_val_changed();
            case DISPLAY_VAL_WAITKEY:
                display = display_val_waitkey( &menuepos );
            break;

            default:
                display = DISPLAY_NORM_INIT;
            break;  
        }
        lastline();
        _delay_ms(50);
    }
}

void lastline() {
    static uint8_t state;
    if ((toggletimer >> TT1s && 0x01 ) == ( state & 0x01 )) { 
        if ( ( state & 0x06 ) == 0 ) state |= 0x06;   // reset to xx xx x1 1x
        state -= 2;
        state ^= 0x01; 
        lcd_gotoxy(1, 1);
        lcd_uint16_unit( val_cur[  state >> 1 ],
                         val_unit[ state >> 1 ] );
    } 
}

uint8_t display_norm_init( void ){
    lcd_gotoxy(0, 0);
    lcd_putc(0xa5);
    lcd_uint16_unit( val_set[ (cur_mode & CUR_MODE_CONST_MASK) >> CUR_MODE_CONST_SHIFT ],
                     val_unit[ (cur_mode & CUR_MODE_CONST_MASK) >> CUR_MODE_CONST_SHIFT ] );
    return DISPLAY_NORM_WAITKEY;
}

uint8_t display_norm_waitkey( void ){
    if( get_key_press ( 1<<KEY0 )) {
        cur_mode ^= CUR_MODE_IS_ACTIVE;
        LED_PORT ^= (1<<LED_PIN);
    }
    if( get_key_long( 1<<KEY1 )){  
        return DISPLAY_VAL_INIT;
    }
    if( get_key_press ( 1<<KEY2 )) {
        return DISPLAY_MENU_INIT;
    }
    return DISPLAY_NORM_WAITKEY;            
}

uint8_t display_menu_init( uint8_t *mp ){
    *mp = 0;
    return DISPLAY_MENU_CHANGED;
}

uint8_t display_menu_changed( const uint8_t *mp ){
    lcd_gotoxy(0, 0);
    draw_menue( *mp );
    return DISPLAY_MENU_WAITKEY;
}

uint8_t display_menu_waitkey( uint8_t *mp ){
    if( get_key_press ( 1<<KEY0 )) {
        *mp = menue_prev( *mp );
        return DISPLAY_MENU_CHANGED;
    }
    if( get_key_press ( 1<<KEY2 )) {
        *mp = menue_next( *mp );
        return DISPLAY_MENU_CHANGED;
    }
    if( get_key_short( 1<<KEY1 )){  
        // TODO: menu item action
    }
    if( get_key_long( 1<<KEY1 )) return DISPLAY_NORM_INIT;
    return DISPLAY_MENU_WAITKEY;
}

uint8_t display_val_init( uint8_t *mp ){
    lcd_command(LCD_DISP_ON_CURSOR_BLINK);
    *mp = 1;
    return DISPLAY_VAL_CHANGED;
}

uint8_t display_val_changed( void ){
    lcd_gotoxy(0,0);
    lcd_putc('#');
    lcd_uint16_unit( val_set[ (cur_mode & CUR_MODE_CONST_MASK) >> CUR_MODE_CONST_SHIFT ],
                     val_unit[ (cur_mode & CUR_MODE_CONST_MASK) >> CUR_MODE_CONST_SHIFT ] );
    return DISPLAY_VAL_WAITKEY;
}

uint8_t display_val_waitkey( uint8_t *mp ){
    // 1-5 => 1,2,4,5,6
    if ( *mp >= 3 ) {
            lcd_gotoxy(*mp +1, 0);
    } else {
            lcd_gotoxy(*mp, 0);
    }
    if( get_key_short( 1<<KEY1 )) {
        *mp = *mp + 1;
        if ( *mp == 6 ) {
            lcd_command(LCD_DISP_ON);
            return DISPLAY_NORM_INIT;
        }
    }
    if( get_key_long( 1<<KEY1 )) {
        lcd_command(LCD_DISP_ON);
        return DISPLAY_NORM_INIT;
    }
    if( get_key_press ( 1<<KEY0 )) {
        val_set[ (cur_mode & CUR_MODE_CONST_MASK) >> CUR_MODE_CONST_SHIFT ] += powten[*mp-1];
        return DISPLAY_VAL_CHANGED;
    }
    if( get_key_press ( 1<<KEY2 )) {
        val_set[ (cur_mode & CUR_MODE_CONST_MASK) >> CUR_MODE_CONST_SHIFT ] -= powten[*mp-1];
        return DISPLAY_VAL_CHANGED;
    }
    return DISPLAY_VAL_WAITKEY;
}
